/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

/*-------------------------------------------------------------------
* access via URL like:
*    http://localhost:8070/resources/applications/{applicationsId}/breakpoints
-------------------------------------------------------------------*/
import zero.ide.services.BreakpointManager
import zero.ide.services.Debugger
import zero.ide.services.DebugUtils
import com.sun.jdi.VirtualMachine
import zero.json.Json
import zero.appbuilder.framework.Framework

/*
*
* @success 200 Returns list of breakpoints (source name and line)
* @format application/json or text/html depending on Accept header
*
*/
def onList() {
	def appId = request.params.applicationsId[]
	def breakpoints = []
	def javaBreakpoints = BreakpointManager.getBreakpoints(appId)
    def phpBreakpoints = breakpointService.getBreakpoints(appId)
    breakpoints.addAll(javaBreakpoints)
    breakpoints.addAll(phpBreakpoints)
	returnJsonObject(breakpoints)
}

/*
* @success 200 Creates a breakpoint; will be set in VM is the VM is running, otherwise 
* will be set shen the VM starts
* @format application/json
*/
def onCreate() {
	def payload = Json.decode(request.input[])
	def enabled = payload.enabled
	if(enabled==null) {
		enabled = true
	}
	def appId = request.params.applicationsId[]
	
	def breakpoint
	// TODO make plugin/factory-oriented
	if (isPhpScript(payload.sourcePath)) {
	    breakpoint = breakpointService.addBreakpoint(appId, payload.sourcePath, (int)payload.line, enabled)
	} else {
	    breakpoint = BreakpointManager.addBreakpoint(appId, payload.sourcePath, (int)payload.line, enabled)
	}
	request.headers.out.Location = getAbsoluteUri("/resources/applications/"+appId+"/breakpoints/"+payload.sourcePath+"/"+payload.line)
	returnJsonObject(breakpoint)
}

/*
 * @success 200 Returns breakpoint source name and line, or "skipAll" collection property
 * @format application/json or text/html depending on Accept header
 */
def onRetrieve() {
	def uri = request.path[]
	def uriParts = uri.split("/")
	def appId = uriParts[3]
	def srcPath = uri.substring(getSourcePathStart(uri, appId))

	if(srcPath=="skipAll") {
		// retrieving skipAll collection property
		returnJsonObject(new Boolean(BreakpointManager.isSkipBreakpointsSet(appId)))
	} else {
	    def breakpoints
	    if (isPhpScript(srcPath)) {
	        breakpoints = breakpointService.getBreakpoints(appId, srcPath)
	    } else {
	        breakpoints = BreakpointManager.getBreakpointsForSource(appId, srcPath) 
	    }
		returnJsonObject(breakpoints)
	}	
}

/* Update 'enabled' state of breakpoint
 * @success 200 Updates breakpoint
 * @format application/json
 */
def onUpdate() {
    
    def payload = Json.decode(request.input[])
	def uri = request.path[]
	def uriParts = uri.split("/")
	def appId = uriParts[3]
	def line = uriParts[uriParts.length-1]
	def srcPath = uri.substring(getSourcePathStart(uri, appId),uri.lastIndexOf(line)-1)
	line = Integer.parseInt(line)
	
	if (isPhpScript(srcPath)) {
	    def bp = breakpointService.getBreakpoint(appId, srcPath, line)      
	    if(bp==null) {
	        logger.FINE {"Could not find breakpoint with sourcePath=" + bp.sourcePath + ", line=" + bp.line}
	        request.status = 404
	        return
	    }
	    // TODO is there a way to update without removing then adding?
	    breakpointService.removeBreakpoint(appId, srcPath, line)
	    bp = breakpointService.addBreakpoint(appId, srcPath, line, payload.enabled)
	    returnJsonObject(bp)
	} else {
	    def bp = BreakpointManager.getBreakpoint(appId, srcPath, line)      
	    if(bp==null) {
	        logger.FINE {"Could not find breakpoint with sourcePath=" + bp.sourcePath + ", line=" + bp.line}
	        request.status = 404
	        return
	    }
	    BreakpointManager.removeBreakpoint(appId, srcPath, line)
	    bp = BreakpointManager.addBreakpoint(appId, srcPath, line, payload.enabled)
	    returnJsonObject(bp)
	}
}

/*
 * Update collectipn properties:
 * skipAll: if true, then skip all breakpoints
 */
def onPutCollection() {
	def appId = request.params.applicationsId[]
	def payload = Json.decode(request.input[])
	logger.INFO { payload.toString() }
	if (payload.skipAll!=null) {
	    
	    // Java/Groovy
		BreakpointManager.setSkipBreakpoints(appId,payload.skipAll)
		
		// PHP
		if (payload.skipAll) {
            breakpointService.disableBreakpoints(appId) 
        } else {
            breakpointService.enableBreakpoints(appId)
        }
	}
		
}

/*
 * @success 200 Deletes breakpoint
 * @format application/json
 */
def onDelete() {
	def uri = request.path[]
	def uriParts = uri.split("/")
	def appId = uriParts[3]
	def line = uriParts[uriParts.length-1]
	def srcPath = uri.substring(getSourcePathStart(uri, appId),uri.lastIndexOf(line)-1)
	
	if (isPhpScript(srcPath)) {
	    breakpointService.removeBreakpoint(appId, srcPath, Integer.parseInt(line))
	} else {
	    BreakpointManager.removeBreakpoint(appId, srcPath, Integer.parseInt(line))
	}
}

def returnJsonObject(object) {
    request.json.output = object
    request.view = "JSON"
    render();
    request.status = 200
}

def getBreakpointService() {
    Framework.createServiceContext().getService('phpdebug')
}

boolean isPhpScript(String sourcePath) {
    return sourcePath.endsWith('.php')
}

def getSourcePathStart(uri, appId) {
    def prefix = "/resources/applications/"+appId+"/breakpoints/"
    return prefix.length()
}