/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

/*-------------------------------------------------------------------
* access via URL like:
*    http://localhost:8070/resources/applications/{applicationsId}/threads
-------------------------------------------------------------------*/

import zero.ide.services.DebugUtils
import zero.ide.services.NotSuspendedException
import zero.json.Json
import com.sun.jdi.VMDisconnectedException
import zero.appbuilder.framework.Framework

/*
*
* @success 200 Lists threads with state info  
* @format application/json or text/html depending on Accept header 
*
*/
def onList() {
    def appId = request.params.applicationsId[]
    
    // get Java/Groovy debugger and validity
	def debugger = DebugUtils.getDebugger()
	if (null == debugger) {
		logger.FINE {DebugUtils.DEBUG_NULL_ERR}
        request.status = 404
        return
    }    
    debugVM = debugger.getVM()
    if (null == debugVM) {
    	logger.FINE {DebugUtils.VM_NULL_ERR}
        request.status = 404
        return
    }
    
    // get PHP debugger
    def phpThreadService = Framework.createServiceContext().getService('phpthreads')
    
    def javaEtag = debugger.getSuspendEventCount()
    // may not exist, so php state defaults to -1
    def phpEtag = phpThreadService ? phpThreadService.getState(appId) : -1
    // remember that etag means *nothing* to clients... just a string... they should not read into the etag value
    def etag = "\"java:${javaEtag};php:${phpEtag}\""
    
    // if neither debugger state has changed, return 304... Not Modified
    if (zcontains("/request/headers/in/If-None-Match")) {
    	if(etag==zget("/request/headers/in/If-None-Match")) {
    		request.status = 304
    		return
    	}
    }
    
    def threadsList = []
    
    /* add PHP threads */
    if (phpThreadService) {
        def phpThreads = phpThreadService.getThreads(appId)
        if (phpThreads) {
            threadsList.addAll(phpThreads)
        }
    }
    /* end PHP adds */
    
    try {
        // add Java/Groovy threads
	    def allThreads = debugVM.allThreads()
	    allThreads.each {
	        threadsList << DebugUtils.getThreadRepresentation(it)
	    }
	    
    } catch (VMDisconnectedException e) {
    	logger.FINE {DebugUtils.VM_DISC_ERR}
    	if(null != debugger) {
    		debugger.cleanup()
    	}
    	request.status = 404
        return
    }
	    
    request.headers.out.Etag = etag.toString();
    returnJsonObject(threadsList)
}

/*
*
* @success 200 Returns thread state info
* @error 404 VM not running or thread not found  
* @format application/json or text/html depending on Accept header 
*
*/
def onRetrieve() {
    
    def appId = request.params.applicationsId[]
    def threadId = request.params.threadsId[]
    threadId = new Integer(threadId)
    
    def thread
    if (0 == threadId) {
        def phpThreadService = Framework.createServiceContext().getService('phpthreads')
        if (phpThreadService) {
            thread = phpThreadService.getThread(appId, threadId)
        }
    } else {
        if (null == DebugUtils.getDebugger()) {
            logger.FINE {DebugUtils.DEBUG_NULL_ERR}
            request.status = 404
            return
        }
        debugVM = DebugUtils.getDebugger().getVM()
        if (null == debugVM) {
            logger.FINE {DebugUtils.VM_NULL_ERR}
            request.status = 404
            return
        }
        
        def threadRef = DebugUtils.getThreadReference()
        if (null == thread) {
            logger.FINE {DebugUtils.THREAD_NULL_ERR}
            request.status = 404
            return
        }
        thread = DebugUtils.getThreadRepresentation(threadRef)
    }
        
    returnJsonObject(thread)
}

/*
*
* @success 200 Executes action on thread 
* including: action: resume, stepOver, stepInto, stepOut, popStack
* @error 404 VM not running or thread not found
* @error 500 Thread not suspended when trying to perform thread action  
* @format application/json 
*/
def onPostMember() {
    def appId = request.params.applicationsId[]
    def threadId = request.params.threadsId[]
    threadId = new Integer(threadId)
    
	def payload = Json.decode(request.input[])
	def action = payload.action
	
	def actionMap = [
	                 'resume' : 'resume',
	                 'stepOver' : 'stepOver',
	                 'stepInto' : 'stepInto',
	                 'stepOut' : 'stepReturn'
	             ]
	
	if (0 == threadId) {
	    // PHP
	    def phpThreadService = Framework.createServiceContext().getService('phpthreads')
        if (phpThreadService) {
            thread = phpThreadService.getThread(appId, threadId)
            // look up action mapping to method and execute
            thread."${actionMap[action]}"()
        }
	} else {
	    // Java/Groovy
	    def debugger = DebugUtils.getDebugger()
        if (null == debugger) {
            logger.FINE {DebugUtils.DEBUG_NULL_ERR}
            request.status = 404
            return
        }
        if(debugger.getVM() == null) {
            logger.FINE {DebugUtils.VM_NULL_ERR}
            request.status = 404
            return
        }
        
        def thread = DebugUtils.getThreadReference()    
        if (null == thread) {
            logger.FINE {DebugUtils.THREAD_NULL_ERR}
            request.status = 404
            return
        }
    
        if(action == "resume") {
            DebugUtils.resume(debugger, thread)            
        } else if(action == "stepOver") {
            DebugUtils.stepOver(debugger, thread)
        } else if(action == "stepInto") {
            DebugUtils.stepInto(debugger, thread)
        } else if(action == "stepOut") {
            DebugUtils.stepOut(debugger, thread)        
        } else if(action == "popStack") {
            DebugUtils.popStack(debugger, thread)
        }
    
        def jsonThread = DebugUtils.getThreadRepresentation(thread)
        
        returnJsonObject(jsonThread)
	}
}

def returnJsonObject(object) {
    request.json.output = object
    request.view = "JSON"
    render();
    request.status = 200
}
